from unittest import TestCase
from Db import Db
from pymongo import MongoClient

URL = u'testurl'
TITLE = u'test title'
TAGS = u'test tags'
BODY = u'test body'

test_client = MongoClient()
test_db = test_client.test
my_db = Db('localhost', 27017, 'test')


class TestDb(TestCase):
    def delete_document(self, my_url=URL):
        """
        Helper function to delete a document from the database
        :param my_url:
        :return: True or False depending if the delete was successful
        """
        result = test_db.test.delete_one({'url': my_url})
        if result is not None:
            return True
        return False

    def create_document(self, my_url=URL, my_title=TITLE, my_tags=TAGS, my_body=BODY):
        """
        Helper function to create a document in the database with parameters
        :param my_url:
        :param my_title:
        :param my_tags:
        :param my_body:
        :return: True or False depending if the insert was successful
        """
        result = test_db.test.insert_one({'url': my_url, 'title': my_title, 'tags': my_tags, 'body': my_body})
        if result is None:
            return False
        return True

    def get_document(self, my_url=URL):
        return test_db.test.find_one({'url': my_url})

    def exists_document(self, my_url=URL):
        """
        Helper function to check the existence of a document in the database.
        :param my_url:
        :return: True or False whether the document exists or not
        """
        result = self.get_document(my_url)
        if result is None:
            return False
        return True

    def test_insert(self):
        """
        Tests if the given content to a document is valid
        :return:
        """
        my_db.insert({'url': URL, 'title': TITLE, 'tags': TAGS, 'body': BODY})
        result = self.get_document()
        assert result is not None
        assert URL in result['url']
        assert TITLE in result['title']
        assert TAGS in result['tags']
        assert BODY in result['body']

    def test_convert(self):
        self.fail()

    def tearDown(self):
        """
        Delete every document in the database
        :return:
        """
        test_db.test.delete_many({})



from collections import OrderedDict
from io import open
import os


class Document(object):
    def __init__(self, path):
        self.path = path
        self.content = ''
        self.meta = ''
        self.body = ''
        self.my_dict = OrderedDict()

    def process_markdown(self):
        """
        Driver that assigns this object's variables with the contents of the
        supplied markdown file, and returns a dictionary with the contents in
        key:value form.
        :return:
        """
        self.get_content()
        self.split_content()
        self.build_request()
        return self.my_dict

    def get_content(self):
        """
        Opens the file and reads it's data to this object's content variable.
        :return:
        """
        with open(self.path, 'r', encoding='utf-8') as f:
            self.content = f.read()

    def split_content(self):
        """
        Splits the content into two categories: meta and body.
        Metadata is at the top of the markdown like this:
        title: my title
        tags: my tags

        And body is the content after the metadata separated by two newlines.
        If there is no separation between metadata and body or there is no
        metadata, it is assumed the data belongs in body.
        :return:
        """
        self.meta = self.content.split('\n\n', 1)
        if len(self.meta) > 1:
            self.body = self.meta[1]
            self.meta = self.meta[0]
            self.meta = self.meta.split('\n')
        else:
            self.body = self.meta[0]
            self.meta = ''

    def build_request(self):
        """
        Create a dictionary in key:value form from the meta and body variables.
        A url key:value is created from the markdown file's name.
        :return:
        """
        for line in self.meta:
            key = line.split(':', 1)[0].strip()
            value = line.split(':', 1)[1].strip()
            self.my_dict[key.lower()] = value.rstrip()
        self.my_dict[u'body'] = self.body.rstrip()
        filename = os.path.basename(self.path)
        self.my_dict[u'url'] = os.path.splitext(filename)[0].rstrip()



# -*- coding: utf-8 -*-
from unittest import TestCase
from io import open
from Document import Document
from collections import OrderedDict


TITLE = u'title: test title'
TAGS = u'tags: test tags'
BODY = u'test body'
CONTENT = TITLE + u'\n' + TAGS + u'\n\n' + BODY
my_doc = Document('test_markdown.md')


class TestDocument(TestCase):
    def create_markdown(self):
        with open('test_markdown.md', 'w+', encoding='utf-8') as f:
            f.write(CONTENT)

    def test_get_content(self):
        self.create_markdown()
        my_doc.get_content()
        assert my_doc.content in CONTENT

    def test_split_content(self):
        my_doc.split_content()
        assert BODY in my_doc.body
        assert TITLE in my_doc.meta
        assert TAGS in my_doc.meta

    def test_build_request(self):
        my_doc.process_markdown()
        test_dict = OrderedDict()
        test_dict[u'title'] = u'test title'
        test_dict[u'tags'] = u'test tags'
        test_dict[u'body'] = u'test body'
        test_dict[u'url'] = 'test_markdown'
        self.assertDictEqual(test_dict, my_doc.my_dict)



# -*- coding: utf-8 -*-
from pymongo import MongoClient
import os
from Document import Document
import argparse


class Db(object):
    def __init__(self, ip='localhost', port=27017, collection_name=''):
        self.collection_name = collection_name
        self.client = MongoClient(ip, port)
        self.db = self.client[self.collection_name]
        self.success_log = []
        self.fail_log = []

    def try_catch(func):
        def wrapped(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception, e:
                print str(e)
        return wrapped

    @try_catch
    def insert(self, my_dict):
        """
        Performs an insert on the MongoDB instance. The dictionary can contain
        anything but must have a url.
        :param my_dict: The processed contents of the markdown file in a
                        dictionary.
        :return: True or False depending if the insert was successful.
        """
        result = self.db[self.collection_name].insert_one(my_dict)
        if result.inserted_id == "":
            return False
        return True

    def convert(self, folder):
        """
        Loops through a supplied folder path looking for markdown files. If it
        finds files with extension .md, the file will be processed into meta
        and body from process_markdown method. After that data is inserted
        into the database.
        :param folder: path to the markdown directory.
        """
        for root, dirs, files in os.walk(folder):
            for file_ in files:
                if file_.endswith('.md'):
                    my_doc = Document(os.path.join(root, file_))
                    data = my_doc.process_markdown()
                    if self.insert(data):
                        self.success_log.append(os.path.join(root, file_))
                    else:
                        self.fail_log.append(os.path.join(root, file_))

    def print_report(self):
        """
        Creates a report of the successes and failures of converting markdown
        files and inserting them into MongoDB.
        """
        def my_print(my_message, my_list):
            if my_list is not None and len(my_list) > 0:
                print(my_message)
                for i in my_list:
                    print("\t{}\n".format(i))

        my_print('Successfully converted and inserted:\n', self.success_log)
        my_print('Could not convert and insert:\n', self.fail_log)


def handle_args():
    """
    Commandline arguments to insert a directory of markdown files into a
    MongoDB instance.
    ip: The ip address of your mongodb instance. Default localhost.
    port: The port of your mongodb instance. Default 27017.
    collection: The name of the collection you want to insert your
                documents into.
    directory: The directory containing your markdown files to insert.
    """
    parser = argparse.ArgumentParser()
    parser.add_argument("ip",
                        help="The ip address of your mongodb \
                         instance. Default localhost.")
    parser.add_argument("port", type=int,
                        help="The port of your mongodb instance. \
                        Default 27017.")
    parser.add_argument("collection",
                        help="The name of the collection you want \
                        to insert your documents into.")
    parser.add_argument("directory",
                        help="The directory containing your markdown \
                        files to insert.")
    args = parser.parse_args()

    collection_name = args.collection
    markdown_directory = args.directory
    ip = args.ip
    port = args.port
    my_db = Db(ip, port, collection_name)
    my_db.convert(markdown_directory)
    my_db.print_report()


if __name__ == '__main__':
    handle_args()



from setuptools import setup


def readme():
    with open('README.rst') as f:
        return f.read()


setup(
    name='markdown_to_mongodb',
    version='0.1.3',
    packages=['markdown_to_mongodb'],
    url='https://github.com/ryanniebling/markdown_to_mongodb',
    license='MIT',
    author='Ryan Niebling',
    author_email='nieblingr1@nku.edu',
    description=readme(),
    longdescription='README.rst',
    install_requires=['pymongo']
)
